unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DskIO, StdCtrls, FileCtrl, Spin, WLT;

type
  TForm1 = class(TForm)
    DriveComboBox: TDriveComboBox;
    Memo: TMemo;
    Memo1: TMemo;
    StartSector: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    nSectors: TSpinEdit;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure DriveComboBoxChange(Sender: TObject);
    procedure StartSectorChange(Sender: TObject);
    procedure nSectorsChange(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    FDiskIO: TDiskIO;
    F: TMemoryStream;
    FBuf: Pointer;
    { Private declarations }
  public
    procedure FillMemo;
    procedure FillMemo1;
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const Filling: Boolean = False;

function IntToHEX(Value: Integer): String;
var B: Byte;
    I: Integer;
begin
   Result := '';
   for I := 8 downto 1 do
     begin
        B := Byte(Value) and $F;
        Value := Value shr 4;
        if B > 9 then B := B-10+$41 else B := B+$30;
        if Result = '' then Result := Char(B) else Insert(Char(B), Result, 1);
     end;
end;

procedure TForm1.FillMemo;
var I: Integer;
begin
   Memo.Lines.Clear;
   Memo.Lines.Add('Logical drive: '+FDiskIO.Drive);
   Memo.Lines.Add('Physical drive: '+IntToStr(FDiskIO.PhysicalDrive));
   Memo.Lines.Add('Total logical sectors: '+IntToStr(FDiskIO.LogicalSectors));
   Memo.Lines.Add('Cylinders: '+IntToStr(FDiskIO.Cylinders));
   Memo.Lines.Add('Sectors per cylinder: '+IntToStr(FDiskIO.PhysicalSectors));
   Memo.Lines.Add('Heads: '+IntToStr(FDiskIO.Heads));
   Memo.Lines.Add('Bytes per sector: '+IntToStr(FDiskIO.BytesPerSector));
   if FDiskIO.FileSystem = fsNone then Memo.Lines.Add('File system: UNKNOWN') else
   if FDiskIO.FileSystem = fsFAT12 then Memo.Lines.Add('File system: FAT12') else
   if FDiskIO.FileSystem = fsFAT16 then Memo.Lines.Add('File system: FAT16') else
      Memo.Lines.Add('File system: FAT32');
   Memo.Lines.Add('Sectors per Cluster: '+IntToStr(FDiskIO.SectorsPerCluster));
   Memo.Lines.Add('Number of copies of FAT: '+IntToStr(FDiskIO.FATCount));
   Memo.Lines.Add('Sectors per FAT: '+IntToStr(FDiskIO.SectorsPerFAT));
   if FDiskIO.FATCount <> 0 then
      for I := 1 to FDiskIO.FATCount do
          Memo.Lines.Add('Start sector for FAT'+IntToStr(I)+': '+IntToStr(FDiskIO.FATSector[I]));
   Memo.Lines.Add('Root DIR Sector: '+IntToStr(FDiskIO.RootDirSector));
   Memo.Lines.Add('Root DIR Cluster: '+IntToStr(FDiskIO.RootDirCluster));
   if FDiskIO.FileSystem <> fsFAT32 then
      Memo.Lines.Add('Root DIR Entries: '+IntToStr(FDiskIO.RootDirEntries));
   Memo.Lines.Add('2-nd Cluster Start Sector: '+IntToStr(FDiskIO.Cluster2Sector));
   Memo.Lines.Add('Ending Cluster: '+IntToStr(FDiskIO.EndingCluster));
   Memo.Lines.Add('SerialVolumeID: '+IntToHEX(FDiskIO.Serial));
   Memo.Lines.Add('Volume Label: '+FDiskIO.VolumeLabel);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
   FDiskIO := TDiskIO.Create;
   F := TMemoryStream.Create;
   FBuf := AllocMem(512);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
   FDiskIO.Free;
   F.Free;
   FreeMem(FBuf);
end;

procedure TForm1.FormShow(Sender: TObject);
begin
   FDiskIO.Drive := DriveComboBox.Drive;
   FillMemo;
   Filling := True;
   StartSector.Value := 0;
   StartSector.MaxValue := FDiskIO.LogicalSectors-1;
   nSectors.Value := 1;
   if FDiskIO.LogicalSectors > 0 then
      nSectors.MaxValue := FDiskIO.LogicalSectors else
      nSectors.MaxValue := 1;
   Filling := False;
   FillMemo1;
end;

procedure TForm1.DriveComboBoxChange(Sender: TObject);
begin
   FormShow(Sender);
end;

function SToHEX(S: String): String;
var B1, B2: Byte;
    I: Integer;
begin
   Result := '';
   if Length(S) = 0 then Exit;
   for I := 1 to Length(S) do
       begin
          B1 := Byte(S[I]);
          B2 := B1 and $F;
          B1 := (B1 shr 4) and $F;
          if B2 > 9 then B2 := B2-10+$41 else B2 := B2+$30;
          if B1 > 9 then B1 := B1-10+$41 else B1 := B1+$30;
          Result := Result+Char(B1)+Char(B2)+' ';
       end;
end;

procedure TForm1.FillMemo1;
var S, N: Integer;
    nSize: Integer;
    SS: String;
begin
   try
     N := nSectors.Value;
   except
     on Exception do Exit;
   end;
   try
     S := StartSector.Value;
   except
     on Exception do Exit;
   end;
   nSize := N*FDiskIO.BytesPerSector;
   Memo1.Clear;
   F.SetSize(nSize);
   F.Seek(0, 0);
   if not FDiskIO.ReadLogicalSector(S, N, F.Memory^, nSize) then
      begin
         MessageBox(0, 'Error reading sectors!', '', MB_OK);
         Exit;
      end;
   FreeMem(FBuf);
   FBuf := AllocMem(nSize);
   F.Seek(0, 0);
   F.Read(FBuf^, nSize);
   F.Seek(0, 0);
   SetLength(SS, 8);
   while F.Size-F.Position > 7 do
     begin
        F.Read(SS[1], 8);
        Memo1.Lines.Add(SToHEX(SS));
     end;
   if F.Size-F.Position <> 0 then
      begin
         SetLength(SS, F.Size-F.Position);
         F.Read(SS[1], F.Size-F.Position);
         Memo1.Lines.Add(SToHEX(SS));
      end;
end;

procedure TForm1.StartSectorChange(Sender: TObject);
var N, S: Integer;
begin
   if Filling then Exit;
   try
     N := nSectors.Value;
   except
     on Exception do Exit;
   end;
   try
     S := StartSector.Value;
   except
     on Exception do Exit;
   end;
   Filling := True;
   if N > FDiskIO.LogicalSectors-S then
      begin
         N := FDiskIO.LogicalSectors-S;
         nSectors.Value := N;
      end;
   nSectors.MaxValue := FDiskIO.LogicalSectors-S;
   Filling := False;
   FillMemo1;
end;

procedure TForm1.nSectorsChange(Sender: TObject);
begin
   if Filling then Exit;
   FillMemo1;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
   Form2.Drive.Drive := DriveComboBox.Drive;
   if Form2.ShowModal <> mrOk then Exit;
   if not Form2.WriteBuffer(FBuf^, F.Size) then
      MessageBox(0, 'Error sector writing!', '', MB_OK);
end;

end.
